#include "bme280.h"

static bool measured_started = false;

typedef struct {
    std::string topic;
    BME280 bme280;
    qthing::bme280_measure_callback_t callback;
} bme280_config_t;

static std::vector<bme280_config_t> cfgs;

#include "esp_log.h"
static void measured(void *ignored) {
    std::string topic;

    while (true) {
        vTaskDelay(1000 / portTICK_PERIOD_MS);

        for (std::vector<bme280_config_t>::iterator it = cfgs.begin(); it != cfgs.end(); ++it) {
            bme280_config_t cfg = *it;

            bme280_reading_data _data = cfg.bme280.readSensorData();
            qthing::bme280_data_t data = {_data.temperature, _data.humidity, _data.pressure};

            if (is_mqtt_connected()) {
                char tmp[16];

                sprintf(tmp, "%0.2f", data.temperature);
                topic = std::string(DEVICE_NAMESPACE) + cfg.topic + std::string("/temperature");
                publish_message(topic, tmp);

                sprintf(tmp, "%0.2f", data.humidity);
                topic = std::string(DEVICE_NAMESPACE) + cfg.topic + std::string("/humidity");
                publish_message(topic, tmp);

                sprintf(tmp, "%0.2f", data.pressure);
                topic = std::string(DEVICE_NAMESPACE) + cfg.topic + std::string("/pressure");
                publish_message(topic, tmp);
            }

            if (cfg.callback != NULL) {
                cfg.callback(data);
            }

            ESP_LOGI((std::string("BME280[") + cfg.topic + std::string("]")).c_str(), "T: %.2f°C  RH: %.2f%%  P: %.2fPa",
                data.temperature,
                data.humidity,
                data.pressure
            );
        }
    }
}

namespace qthing {

    void add_bme280(gpio_num_t sda, gpio_num_t scl, std::string topic, qthing::bme280_measure_callback_t callback, uint8_t address) {
        BME280 bme280 = BME280(address);
        bme280.setDebug(false);
        bme280.init(sda, scl);

        bme280_config_t cfg = {
            topic,
            bme280,
            callback
        };

        cfgs.push_back(cfg);

        if (!measured_started) {
            measured_started = true;
            xTaskCreate(measured, "bme280_measured", 10000, NULL, 1, NULL);
        }
    }
    
    void add_bme280(gpio_num_t sda, gpio_num_t scl, std::string topic, uint8_t address) {
        add_bme280(sda, scl, topic, NULL, address);
    }
}
